from m3_gar.base_models import (
    NormativeDocs as BaseNormativeDocs,
    NormativeDocsKinds as BaseNormativeDocsKinds,
    NormativeDocsTypes as BaseNormativeDocsTypes,
)
from m3_gar.models.util import (
    RegionCodeModelMixin,
    make_fk,
)


__all__ = ['NormativeDocsKinds', 'NormativeDocsTypes', 'NormativeDocs']


class NormativeDocsKinds(BaseNormativeDocsKinds):
    """
    Сведения по видам нормативных документов
    """
    class Meta:
        verbose_name = 'Вид нормативного документа'
        verbose_name_plural = 'Виды нормативных документов'

    def __str__(self):
        return self.name


class NormativeDocsTypes(BaseNormativeDocsTypes):
    """
    Сведения по типам нормативных документов
    """
    class Meta:
        verbose_name = 'Тип нормативного документа'
        verbose_name_plural = 'Типы нормативных документов'

    def __str__(self):
        return self.name


class NormativeDocs(BaseNormativeDocs, RegionCodeModelMixin):
    """
    Сведения о нормативном документе, являющемся основанием присвоения
    адресному элементу наименования
    """

    # type = tt(to=NormativeDocsTypes, on_delete=models.CASCADE)

    class Meta:
        verbose_name = 'Нормативный документ'
        verbose_name_plural = 'Нормативные документы'

    def __str__(self):
        return self.name


make_fk(NormativeDocs, 'type', to=NormativeDocsTypes)
make_fk(NormativeDocs, 'kind', to=NormativeDocsKinds)



# Generated by Django 3.2.4 on 2021-07-06 12:37

import django.db.models.deletion
from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AddrObjDivision',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Уникальный идентификатор записи. Ключевое поле')),
                ('parentid', models.BigIntegerField(verbose_name='Родительский ID')),
                ('childid', models.BigIntegerField(verbose_name='Дочерний ID')),
                ('changeid', models.BigIntegerField(verbose_name='ID изменившей транзакции')),
            ],
            options={
                'verbose_name': 'Операция переподчинения',
                'verbose_name_plural': 'Операции переподчинения',
            },
        ),
        migrations.CreateModel(
            name='AddrObjTypes',
            fields=[
                ('id', models.IntegerField(primary_key=True, serialize=False, verbose_name='Идентификатор записи')),
                ('level', models.IntegerField(verbose_name='Уровень адресного объекта')),
                ('shortname', models.CharField(max_length=50, verbose_name='Краткое наименование типа объекта')),
                ('name', models.CharField(max_length=250, verbose_name='Полное наименование типа объекта')),
                ('desc', models.CharField(blank=True, max_length=250, null=True, verbose_name='Описание')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Начало действия записи')),
                ('enddate', models.DateField(verbose_name='Окончание действия записи')),
                ('isactive', models.BooleanField(verbose_name='Статус активности')),
            ],
            options={
                'verbose_name': 'Тип адресного объекта',
                'verbose_name_plural': 'Типы адресных объектов',
            },
        ),
        migrations.CreateModel(
            name='ApartmentTypes',
            fields=[
                ('id', models.IntegerField(primary_key=True, serialize=False, verbose_name='Идентификатор типа (ключ)')),
                ('name', models.CharField(max_length=50, verbose_name='Наименование')),
                ('shortname', models.CharField(blank=True, max_length=50, null=True, verbose_name='Краткое наименование')),
                ('desc', models.CharField(blank=True, max_length=250, null=True, verbose_name='Описание')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Начало действия записи')),
                ('enddate', models.DateField(verbose_name='Окончание действия записи')),
                ('isactive', models.BooleanField(verbose_name='Статус активности')),
            ],
            options={
                'verbose_name': 'Тип помещения',
                'verbose_name_plural': 'Типы помещений',
            },
        ),
        migrations.CreateModel(
            name='HouseTypes',
            fields=[
                ('id', models.IntegerField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('name', models.CharField(max_length=50, verbose_name='Наименование')),
                ('shortname', models.CharField(blank=True, max_length=50, null=True, verbose_name='Краткое наименование')),
                ('desc', models.CharField(blank=True, max_length=250, null=True, verbose_name='Описание')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Начало действия записи')),
                ('enddate', models.DateField(verbose_name='Окончание действия записи')),
                ('isactive', models.BooleanField(verbose_name='Статус активности')),
            ],
            options={
                'verbose_name': 'Тип дома',
                'verbose_name_plural': 'Типы домов',
            },
        ),
        migrations.CreateModel(
            name='NormativeDocsKinds',
            fields=[
                ('id', models.IntegerField(primary_key=True, serialize=False, verbose_name='Идентификатор записи')),
                ('name', models.CharField(max_length=500, verbose_name='Наименование')),
            ],
            options={
                'verbose_name': 'Вид нормативного документа',
                'verbose_name_plural': 'Виды нормативных документов',
            },
        ),
        migrations.CreateModel(
            name='NormativeDocsTypes',
            fields=[
                ('id', models.IntegerField(primary_key=True, serialize=False, verbose_name='Идентификатор записи')),
                ('name', models.CharField(max_length=500, verbose_name='Наименование')),
                ('startdate', models.DateField(verbose_name='Дата начала действия записи')),
                ('enddate', models.DateField(verbose_name='Дата окончания действия записи')),
            ],
            options={
                'verbose_name': 'Тип нормативного документа',
                'verbose_name_plural': 'Типы нормативных документов',
            },
        ),
        migrations.CreateModel(
            name='ObjectLevels',
            fields=[
                ('level', models.IntegerField(primary_key=True, serialize=False, verbose_name='Уникальный идентификатор записи. Ключевое поле. Номер уровня объекта')),
                ('name', models.CharField(max_length=250, verbose_name='Наименование')),
                ('shortname', models.CharField(blank=True, max_length=50, null=True, verbose_name='Краткое наименование')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Начало действия записи')),
                ('enddate', models.DateField(verbose_name='Окончание действия записи')),
                ('isactive', models.BooleanField(verbose_name='Признак действующего адресного объекта')),
            ],
            options={
                'verbose_name': 'Уровень адресного объекта',
                'verbose_name_plural': 'Уровни адресных объектов',
            },
        ),
        migrations.CreateModel(
            name='OperationTypes',
            fields=[
                ('id', models.IntegerField(primary_key=True, serialize=False, verbose_name='Идентификатор статуса (ключ)')),
                ('name', models.CharField(max_length=100, verbose_name='Наименование')),
                ('shortname', models.CharField(blank=True, max_length=100, null=True, verbose_name='Краткое наименование')),
                ('desc', models.CharField(blank=True, max_length=250, null=True, verbose_name='Описание')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Начало действия записи')),
                ('enddate', models.DateField(verbose_name='Окончание действия записи')),
                ('isactive', models.BooleanField(verbose_name='Статус активности')),
            ],
            options={
                'verbose_name': 'Статус действия',
                'verbose_name_plural': 'Статусы действия',
            },
        ),
        migrations.CreateModel(
            name='ParamTypes',
            fields=[
                ('id', models.IntegerField(primary_key=True, serialize=False, verbose_name='Идентификатор типа параметра (ключ)')),
                ('name', models.CharField(max_length=50, verbose_name='Наименование')),
                ('code', models.CharField(max_length=50, verbose_name='Краткое наименование')),
                ('desc', models.CharField(blank=True, max_length=120, null=True, verbose_name='Описание')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Начало действия записи')),
                ('enddate', models.DateField(verbose_name='Окончание действия записи')),
                ('isactive', models.BooleanField(verbose_name='Статус активности')),
            ],
            options={
                'verbose_name': 'Тип параметра',
                'verbose_name_plural': 'Типы параметров',
            },
        ),
        migrations.CreateModel(
            name='ReestrObjects',
            fields=[
                ('objectid', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Уникальный идентификатор объекта')),
                ('createdate', models.DateField(verbose_name='Дата создания')),
                ('changeid', models.BigIntegerField(verbose_name='ID изменившей транзакции')),
                ('updatedate', models.DateField(verbose_name='Дата обновления')),
                ('objectguid', models.CharField(max_length=36, verbose_name='GUID объекта')),
                ('isactive', models.BooleanField(verbose_name='Признак действующего объекта (1 - действующий, 0 - не действующий)')),
                ('levelid', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.objectlevels', verbose_name='Уровень объекта')),
            ],
            options={
                'verbose_name': 'Идентификатор адресного элемента',
                'verbose_name_plural': 'Идентификаторы адресных элементов',
            },
        ),
        migrations.CreateModel(
            name='RoomTypes',
            fields=[
                ('id', models.IntegerField(primary_key=True, serialize=False, verbose_name='Идентификатор типа (ключ)')),
                ('name', models.CharField(max_length=100, verbose_name='Наименование')),
                ('shortname', models.CharField(blank=True, max_length=50, null=True, verbose_name='Краткое наименование')),
                ('desc', models.CharField(blank=True, max_length=250, null=True, verbose_name='Описание')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Начало действия записи')),
                ('enddate', models.DateField(verbose_name='Окончание действия записи')),
                ('isactive', models.BooleanField(verbose_name='Статус активности')),
            ],
            options={
                'verbose_name': 'Тип комнаты',
                'verbose_name_plural': 'Типы комнат',
            },
        ),
        migrations.CreateModel(
            name='Version',
            fields=[
                ('ver', models.IntegerField(primary_key=True, serialize=False)),
                ('date', models.DateField(blank=True, db_index=True, null=True)),
                ('dumpdate', models.DateField(db_index=True)),
                ('complete_xml_url', models.CharField(max_length=255)),
                ('delta_xml_url', models.CharField(blank=True, max_length=255, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Steads',
            fields=[
                ('id', models.IntegerField(primary_key=True, serialize=False, verbose_name='Уникальный идентификатор записи. Ключевое поле')),
                ('objectguid', models.CharField(max_length=36, verbose_name='Глобальный уникальный идентификатор адресного объекта типа UUID')),
                ('changeid', models.IntegerField(verbose_name='ID изменившей транзакции')),
                ('number', models.CharField(max_length=250, verbose_name='Номер земельного участка')),
                ('previd', models.IntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с предыдущей исторической записью')),
                ('nextid', models.IntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с последующей исторической записью')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Начало действия записи')),
                ('enddate', models.DateField(verbose_name='Окончание действия записи')),
                ('isactual', models.BooleanField(verbose_name='Статус актуальности адресного объекта ФИАС')),
                ('isactive', models.BooleanField(verbose_name='Признак действующего адресного объекта')),
                ('objectid', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.reestrobjects', verbose_name='Глобальный уникальный идентификатор объекта типа INTEGER')),
                ('opertypeid', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.operationtypes', verbose_name='Статус действия над записью – причина появления записи')),
            ],
            options={
                'verbose_name': 'Земельный участок',
                'verbose_name_plural': 'Земельные участки',
            },
        ),
        migrations.CreateModel(
            name='Status',
            fields=[
                ('table', models.CharField(max_length=25, primary_key=True, serialize=False)),
                ('ver', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='m3_gar.version')),
            ],
        ),
        migrations.CreateModel(
            name='Rooms',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Уникальный идентификатор записи. Ключевое поле')),
                ('objectguid', models.CharField(max_length=36, verbose_name='Глобальный уникальный идентификатор адресного объекта типа UUID')),
                ('changeid', models.BigIntegerField(verbose_name='ID изменившей транзакции')),
                ('number', models.CharField(max_length=50, verbose_name='Номер комнаты или офиса')),
                ('previd', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с предыдущей исторической записью')),
                ('nextid', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с последующей исторической записью')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Начало действия записи')),
                ('enddate', models.DateField(verbose_name='Окончание действия записи')),
                ('isactual', models.BooleanField(verbose_name='Статус актуальности адресного объекта ФИАС')),
                ('isactive', models.BooleanField(verbose_name='Признак действующего адресного объекта')),
                ('objectid', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.reestrobjects', verbose_name='Глобальный уникальный идентификатор объекта типа INTEGER')),
                ('opertypeid', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.operationtypes', verbose_name='Статус действия над записью – причина появления записи')),
                ('roomtype', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.roomtypes', verbose_name='Тип комнаты или офиса')),
            ],
            options={
                'verbose_name': 'Комната',
                'verbose_name_plural': 'Комнаты',
            },
        ),
        migrations.CreateModel(
            name='Param',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Идентификатор записи')),
                ('objectid', models.BigIntegerField(verbose_name='Глобальный уникальный идентификатор адресного объекта ')),
                ('changeid', models.BigIntegerField(blank=True, null=True, verbose_name='ID изменившей транзакции')),
                ('changeidend', models.BigIntegerField(verbose_name='ID завершившей транзакции')),
                ('value', models.CharField(max_length=8000, verbose_name='Значение параметра')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Дата начала действия записи')),
                ('enddate', models.DateField(verbose_name='Дата окончания действия записи')),
                ('typeid', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.paramtypes', verbose_name='Тип параметра')),
            ],
            options={
                'verbose_name': 'Параметр',
                'verbose_name_plural': 'Параметры',
            },
        ),
        migrations.CreateModel(
            name='NormativeDocs',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Уникальный идентификатор документа')),
                ('name', models.CharField(max_length=8000, verbose_name='Наименование документа')),
                ('date', models.DateField(verbose_name='Дата документа')),
                ('number', models.CharField(max_length=150, verbose_name='Номер документа')),
                ('updatedate', models.DateField(verbose_name='Дата обновления')),
                ('orgname', models.CharField(blank=True, max_length=255, null=True, verbose_name='Наименование органа создвшего нормативный документ')),
                ('regnum', models.CharField(blank=True, max_length=100, null=True, verbose_name='Номер государственной регистрации')),
                ('regdate', models.DateField(blank=True, null=True, verbose_name='Дата государственной регистрации')),
                ('accdate', models.DateField(blank=True, null=True, verbose_name='Дата вступления в силу нормативного документа')),
                ('comment', models.CharField(blank=True, max_length=8000, null=True, verbose_name='Комментарий')),
                ('kind', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.normativedocskinds', verbose_name='Вид документа')),
                ('type', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.normativedocstypes', verbose_name='Тип документа')),
            ],
            options={
                'verbose_name': 'Нормативный документ',
                'verbose_name_plural': 'Нормативные документы',
            },
        ),
        migrations.CreateModel(
            name='MunHierarchy',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Уникальный идентификатор записи. Ключевое поле')),
                ('changeid', models.BigIntegerField(verbose_name='ID изменившей транзакции')),
                ('oktmo', models.CharField(blank=True, max_length=11, null=True, verbose_name='Код ОКТМО')),
                ('previd', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с предыдущей исторической записью')),
                ('nextid', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с последующей исторической записью')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Начало действия записи')),
                ('enddate', models.DateField(verbose_name='Окончание действия записи')),
                ('isactive', models.BooleanField(verbose_name='Признак действующего адресного объекта')),
                ('objectid', models.OneToOneField(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.reestrobjects', verbose_name='Глобальный уникальный идентификатор адресного объекта ')),
                ('parentobjid', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.reestrobjects', verbose_name='Идентификатор родительского объекта')),
            ],
            options={
                'verbose_name': 'Иерархия в муниципальном делении',
                'verbose_name_plural': 'Иерархия в муниципальном делении',
            },
        ),
        migrations.CreateModel(
            name='Houses',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Уникальный идентификатор записи. Ключевое поле')),
                ('objectguid', models.CharField(max_length=36, verbose_name='Глобальный уникальный идентификатор адресного объекта типа UUID')),
                ('changeid', models.BigIntegerField(verbose_name='ID изменившей транзакции')),
                ('housenum', models.CharField(blank=True, max_length=50, null=True, verbose_name='Основной номер дома')),
                ('addnum1', models.CharField(blank=True, max_length=50, null=True, verbose_name='Дополнительный номер дома 1')),
                ('addnum2', models.CharField(blank=True, max_length=50, null=True, verbose_name='Дополнительный номер дома 1')),
                ('previd', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с предыдущей исторической записью')),
                ('nextid', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с последующей исторической записью')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Начало действия записи')),
                ('enddate', models.DateField(verbose_name='Окончание действия записи')),
                ('isactual', models.BooleanField(verbose_name='Статус актуальности адресного объекта ФИАС')),
                ('isactive', models.BooleanField(verbose_name='Признак действующего адресного объекта')),
                ('addtype1', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.housetypes', verbose_name='Дополнительный тип дома 1')),
                ('addtype2', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.housetypes', verbose_name='Дополнительный тип дома 2')),
                ('housetype', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.housetypes', verbose_name='Основной тип дома')),
                ('objectid', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.reestrobjects', verbose_name='Глобальный уникальный идентификатор объекта типа INTEGER')),
                ('opertypeid', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.operationtypes', verbose_name='Статус действия над записью – причина появления записи')),
            ],
            options={
                'verbose_name': 'Номер дома',
                'verbose_name_plural': 'Номера домов',
            },
        ),
        migrations.CreateModel(
            name='ChangeHistory',
            fields=[
                ('changeid', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='ID изменившей транзакции')),
                ('objectid', models.BigIntegerField(verbose_name='Уникальный ID объекта')),
                ('adrobjectid', models.CharField(max_length=36, verbose_name='Уникальный ID изменившей транзакции (GUID)')),
                ('ndocid', models.BigIntegerField(blank=True, null=True, verbose_name='ID документа')),
                ('changedate', models.DateField(verbose_name='Дата изменения')),
                ('opertypeid', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.operationtypes', verbose_name='Тип операции')),
            ],
            options={
                'verbose_name': 'История изменений',
                'verbose_name_plural': 'История изменений',
            },
        ),
        migrations.CreateModel(
            name='Carplaces',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Уникальный идентификатор записи. Ключевое поле')),
                ('objectguid', models.CharField(max_length=36, verbose_name='Глобальный уникальный идентификатор адресного объекта типа UUID')),
                ('changeid', models.BigIntegerField(verbose_name='ID изменившей транзакции')),
                ('number', models.CharField(max_length=50, verbose_name='Номер машиноместа')),
                ('previd', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с предыдущей исторической записью')),
                ('nextid', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с последующей исторической записью')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Начало действия записи')),
                ('enddate', models.DateField(verbose_name='Окончание действия записи')),
                ('isactual', models.BooleanField(verbose_name='Статус актуальности адресного объекта ФИАС')),
                ('isactive', models.BooleanField(verbose_name='Признак действующего адресного объекта')),
                ('objectid', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.reestrobjects', verbose_name='Глобальный уникальный идентификатор объекта типа INTEGER')),
                ('opertypeid', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.operationtypes', verbose_name='Статус действия над записью – причина появления записи')),
            ],
            options={
                'verbose_name': 'Машино-место',
                'verbose_name_plural': 'Машино-места',
            },
        ),
        migrations.CreateModel(
            name='Apartments',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Уникальный идентификатор записи. Ключевое поле')),
                ('objectguid', models.CharField(max_length=36, verbose_name='Глобальный уникальный идентификатор адресного объекта типа UUID')),
                ('changeid', models.BigIntegerField(verbose_name='ID изменившей транзакции')),
                ('number', models.CharField(max_length=50, verbose_name='Номер комнаты')),
                ('previd', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с предыдущей исторической записью')),
                ('nextid', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с последующей исторической записью')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Начало действия записи')),
                ('enddate', models.DateField(verbose_name='Окончание действия записи')),
                ('isactual', models.BooleanField(verbose_name='Статус актуальности адресного объекта ФИАС')),
                ('isactive', models.BooleanField(verbose_name='Признак действующего адресного объекта')),
                ('aparttype', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.apartmenttypes', verbose_name='Тип комнаты')),
                ('objectid', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.reestrobjects', verbose_name='Глобальный уникальный идентификатор объекта типа INTEGER')),
                ('opertypeid', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.operationtypes', verbose_name='Статус действия над записью – причина появления записи')),
            ],
            options={
                'verbose_name': 'Помещение',
                'verbose_name_plural': 'Помещения',
            },
        ),
        migrations.CreateModel(
            name='AdmHierarchy',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Уникальный идентификатор записи. Ключевое поле')),
                ('changeid', models.BigIntegerField(verbose_name='ID изменившей транзакции')),
                ('regioncode', models.CharField(blank=True, max_length=4, null=True, verbose_name='Код региона')),
                ('areacode', models.CharField(blank=True, max_length=4, null=True, verbose_name='Код района')),
                ('citycode', models.CharField(blank=True, max_length=4, null=True, verbose_name='Код города')),
                ('placecode', models.CharField(blank=True, max_length=4, null=True, verbose_name='Код населенного пункта')),
                ('plancode', models.CharField(blank=True, max_length=4, null=True, verbose_name='Код ЭПС')),
                ('streetcode', models.CharField(blank=True, max_length=4, null=True, verbose_name='Код улицы')),
                ('previd', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с предыдущей исторической записью')),
                ('nextid', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с последующей исторической записью')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Начало действия записи')),
                ('enddate', models.DateField(verbose_name='Окончание действия записи')),
                ('isactive', models.BooleanField(verbose_name='Признак действующего адресного объекта')),
                ('objectid', models.OneToOneField(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.reestrobjects', verbose_name='Глобальный уникальный идентификатор объекта')),
                ('parentobjid', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.reestrobjects', verbose_name='Идентификатор родительского объекта')),
            ],
            options={
                'verbose_name': 'Иерархия в административном делении',
                'verbose_name_plural': 'Иерархия в административном делении',
            },
        ),
        migrations.CreateModel(
            name='AddrObj',
            fields=[
                ('id', models.BigIntegerField(primary_key=True, serialize=False, verbose_name='Уникальный идентификатор записи. Ключевое поле')),
                ('objectguid', models.CharField(max_length=36, verbose_name='Глобальный уникальный идентификатор адресного объекта типа UUID')),
                ('changeid', models.BigIntegerField(verbose_name='ID изменившей транзакции')),
                ('name', models.CharField(max_length=250, verbose_name='Наименование')),
                ('typename', models.CharField(max_length=50, verbose_name='Краткое наименование типа объекта')),
                ('level', models.CharField(max_length=10, verbose_name='Уровень адресного объекта')),
                ('previd', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с предыдущей исторической записью')),
                ('nextid', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор записи связывания с последующей исторической записью')),
                ('updatedate', models.DateField(verbose_name='Дата внесения (обновления) записи')),
                ('startdate', models.DateField(verbose_name='Начало действия записи')),
                ('enddate', models.DateField(verbose_name='Окончание действия записи')),
                ('isactual', models.BooleanField(verbose_name='Статус актуальности адресного объекта ФИАС')),
                ('isactive', models.BooleanField(verbose_name='Признак действующего адресного объекта')),
                ('objectid', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.reestrobjects', verbose_name='Глобальный уникальный идентификатор адресного объекта типа INTEGER')),
                ('opertypeid', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='m3_gar.operationtypes', verbose_name='Статус действия над записью – причина появления записи')),
            ],
            options={
                'verbose_name': 'Адресообразующий элемент',
                'verbose_name_plural': 'Адресообразующие элементы',
            },
        ),
    ]



import os

from django.conf import (
    settings,
)
from django.core.exceptions import (
    ImproperlyConfigured,
)
from django.db.utils import (
    DEFAULT_DB_ALIAS,
)


DATABASE_ALIAS = getattr(settings, 'GAR_DATABASE_ALIAS', DEFAULT_DB_ALIAS)

if DATABASE_ALIAS not in settings.DATABASES:
    raise ImproperlyConfigured(f'GAR: database alias `{DATABASE_ALIAS}` was not found in DATABASES')
elif DATABASE_ALIAS != DEFAULT_DB_ALIAS and 'm3_gar.routers.GARRouter' not in settings.DATABASE_ROUTERS:
    raise ImproperlyConfigured(
        'GAR: to use external database add `m3_gar.routers.GARRouter` '
        'into `DATABASE_ROUTERS` list in your settings.py'
    )

SUGGEST_BACKEND = getattr(settings, 'GAR_SUGGEST_BACKEND', 'gar.suggest.backends.noop')
SUGGEST_VIEW = getattr(settings, 'GAR_SUGGEST_VIEW', 'gar:suggest')
SUGGEST_AREA_VIEW = getattr(settings, 'GAR_SUGGEST_AREA_VIEW', 'gar:suggest-area')

# SUDS Proxy Support
_http_proxy = os.environ.get('http_proxy')
_https_proxy = os.environ.get('https_proxy')

PROXY = {}
if _http_proxy:
    PROXY['http'] = _http_proxy
if _https_proxy:
    PROXY['https'] = _https_proxy



from m3_gar.importer.source.tablelist import (
    TableList,
)
from m3_gar.importer.source.wrapper import (
    DirectoryWrapper,
)


class DirectoryTableList(TableList):
    wrapper_class = DirectoryWrapper

    def load_data(self, source):
        return self.wrapper_class(source=source, is_temporary=False)



# Generated by Django 2.2.28 on 2022-05-13 08:25

import django.contrib.postgres.indexes
import m3_gar.models.indexes

from django.db import migrations
from django.contrib.postgres.operations import TrigramExtension


class Migration(migrations.Migration):

    dependencies = [
        ('m3_gar', '0012_auto_20220415_1452'),
    ]

    operations = [
        TrigramExtension(),
        migrations.RemoveIndex(
            model_name='admhierarchy',
            name='m3_gar_admh_name_wi_5f5f5a_gin',
        ),
        migrations.RemoveIndex(
            model_name='munhierarchy',
            name='m3_gar_munh_name_wi_380974_gin',
        ),
        migrations.AddIndex(
            model_name='addrobj',
            index=django.contrib.postgres.indexes.BTreeIndex(fields=['level'], name='m3_gar_addr_level_5ff8c6_btree'),
        ),
        migrations.AddIndex(
            model_name='admhierarchy',
            index=m3_gar.models.indexes.UpperGinIndex(fastupdate=True, fields=['name_with_parents'], name='adm_name_with_parents_gin', opclasses=['gin_trgm_ops']),
        ),
        migrations.AddIndex(
            model_name='munhierarchy',
            index=m3_gar.models.indexes.UpperGinIndex(fastupdate=True, fields=['name_with_parents'], name='mun_name_with_parents_gin', opclasses=['gin_trgm_ops']),
        ),
    ]



import logging
import sys

from django.core.management import (
    BaseCommand,
)
from m3_gar_constants import (
    GAR_LEVELS_PLACE,
    CODE_PARAM_TYPES_OFFICIAL,
    GAR_LEVELS_STREET,
)
from m3_gar.models import (
    AddrObj,
    AddrObjParams,
    ParamTypes,
)
from m3_gar.models.hierarchy import (
    MunHierarchy,
    AdmHierarchy,
)


CACHE_OBJECTS = {}


def create_logger():
    """Возвращает объект для логгирования выполнения команды"""
    logger = logging.getLogger()
    logger.setLevel(logging.INFO)

    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(logging.INFO)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    return logger


logger = create_logger()


def chunks(lst, n):
    """Разбивает лист на несколько листов размером n"""
    for i in range(0, len(lst), n):
        yield lst[i:i + n]


def add_object_to_cache(objectid, name):
    """Кэширование объектов запроса для сокращения количества запросов в БД"""
    CACHE_OBJECTS[objectid] = name


class Command(BaseCommand):
    """
    Команда для обновления поля name_with_parents в моделях иерархий и поля name_with_typename в модели AddrObj

    Доступные аргументы:
    --parents - заполняем поля name_with_parents
        --adm - поля обновляются для административной иерархии
        --guids - можно через запятую указать guid-ы объектов, для которых нужно обновить поле
        --levels - можно через запятую указать уровни объектов, для которых нужно обновить поле
    --typenames - заполняем поля name_with_typename для уровней 7 и 8
        --guids_typenames - можно через запятую указать guid-ы объектов, для которых нужно обновить поле
    """
    help = 'Утилита для заполнения дополнительного поля name_with_parents в модели Hierarchy и ' \
           'поля name_with_typename в модели AddrObj'

    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument(
            '--parents',
            action='store_true',
            default=None,
            help='Fill column name_with_parents',
        )

        parser.add_argument(
            '--typenames',
            action='store_true',
            default=None,
            help='Fill column name_with_typename',
        )

        parser.add_argument(
            '--adm',
            action='store_true',
            default=None,
            help='Change hierarchy model to AdmHierarchy',
        )

        parser.add_argument(
            '--guids',
            default=None,
            help='Add objects guid for filter columns for adding name_with_parents column',
        )

        parser.add_argument(
            '--guids_typenames',
            default=None,
            help='Add objects guid for filter for adding name_with_typename column',
        )

        parser.add_argument(
            '--levels',
            default=None,
            help='Add objects level for filter columns for adding data',
        )

    @staticmethod
    def get_name_with_full_typename(obj, official_type):

        addr_obj_name = ''

        try:
            addr_obj = AddrObj.objects.get(
                objectid=obj.objectid_id,
                isactual=True,
            )

            official_param = None
            if official_type:
                official_param = AddrObjParams.objects.filter(
                    objectid=obj.objectid_id,
                    typeid=official_type,
                )
        except Exception as err:
            logger.info(f'В ходе выполнения команды возникла ошибка: {err}')
        else:
            if addr_obj:
                type_name = addr_obj.type_full_name
                # Если Респ, то Республика, а если г., то город
                if addr_obj.typename and addr_obj.typename[0].islower():
                    type_name = type_name.lower()

                # Делаем так, чтобы не "область Кировская", а "Кировская область"
                # Впрочем, если есть официальное название, то используем его.
                # Перестановки остаются на уровне районов, городов и т.д.
                if official_param:
                    for actual_param in official_param:
                        # Может быть два официальных названия: Пермский край и Пермская область. Используем то,
                        # которое используется на данный момент. При этом заранее присваиваем наименование,
                        # чтобы не возникла ошибка, когда в рамках цикла по какой-либо причине условие не выполнится.
                        official_name = actual_param.value
                        if type_name.lower() in actual_param.value.lower():
                            official_name = actual_param.value
                            break

                    addr_obj_name = f'{official_name}'
                elif addr_obj.is_prefix_type:
                    addr_obj_name = f'{type_name} {addr_obj.name}'
                else:
                    addr_obj_name = f'{addr_obj.name} {type_name}'

            return addr_obj_name, addr_obj.level

    def get_name_with_parents(self, obj, official_type, max_level_value):

        result_parts = []

        for item in obj.get_ancestors(include_self=True):
            object_id = item.objectid_id

            if object_id in CACHE_OBJECTS:
                addr_obj_name = CACHE_OBJECTS[object_id]

            else:
                addr_obj_name, addr_obj_level = self.get_name_with_full_typename(item, official_type)
                if addr_obj_level != max_level_value:
                    CACHE_OBJECTS[object_id] = addr_obj_name

            result_parts.append(addr_obj_name)

        result = ', '.join(result_parts)

        return result

    @staticmethod
    def update_db_data(
            model,
            data_for_update,
            column_name,
            fill_column_function,
            max_level_value=None,
    ):

        logger.info(f'Запущено обновление полей {column_name} для модели {model.__name__}')

        official_type = ParamTypes.objects.filter(code=CODE_PARAM_TYPES_OFFICIAL, isactive=True).get()
        data = data_for_update.order_by('id')
        data_length = data.count()
        length_checked_lines = 0

        for index, lines_list in enumerate(chunks(data, 500), start=1):
            lines_for_update = []

            for line in lines_list:
                if column_name == 'name_with_parents':
                    column_value = fill_column_function(line, official_type, max_level_value)
                    if line.name_with_parents != column_value:
                        line.name_with_parents = column_value
                        lines_for_update.append(line)
                elif column_name == 'name_with_typename':
                    column_value = fill_column_function(line, official_type)[0]
                    if line.name_with_typename != column_value:
                        line.name_with_typename = column_value
                        lines_for_update.append(line)

            length_checked_lines += len(lines_list)
            logger.info(
                f'На необходимость обновления проверено {length_checked_lines} записей из {data_length} записей'
            )
            if lines_for_update:
                logger.info(f'Будет обработано {len(lines_for_update)} записей')

                model.objects.bulk_update(
                    objs=lines_for_update,
                    fields=(
                        column_name,
                    ),
                )

                logger.info(f'{len(lines_for_update)} полей записаны успешно')
            else:
                logger.info(f'Записи не нуждаются в обновлении')

    def handle(self, *args, parents, typenames, adm, guids, guids_typenames, levels, **options):

        if parents:
            # Муниципальная модель иерархии по умолчанию
            hierarchy_model = MunHierarchy
            if adm:
                hierarchy_model = AdmHierarchy

            levels_list = GAR_LEVELS_PLACE

            if guids:
                guids_list = guids.split(',')
                object_ids = AddrObj.objects.filter(
                    objectguid__in=guids_list, level__in=levels_list
                ).values_list('objectid', flat=True)
                hierarchy_data = hierarchy_model.objects.filter(objectid__in=object_ids)

            elif levels:
                levels_list = levels.split(',')
                object_ids = AddrObj.objects.filter(level__in=levels_list).values_list('objectid', flat=True)
                hierarchy_data = hierarchy_model.objects.filter(objectid__in=object_ids)

            else:
                object_ids = AddrObj.objects.filter(level__in=levels_list).values_list('objectid', flat=True)
                hierarchy_data = hierarchy_model.objects.filter(objectid__in=object_ids)

            # Максимальное значение уровня, чтобы не кэшировать объекты этого уровня
            max_level_value = str(max(levels_list))

            self.update_db_data(
                model=hierarchy_model,
                data_for_update=hierarchy_data,
                column_name='name_with_parents',
                fill_column_function=self.get_name_with_parents,
                max_level_value=max_level_value,
            )

        if typenames:
            if guids_typenames:
                guids_list = guids_typenames.split(',')
                typenames_data = AddrObj.objects.filter(
                    isactive=True,
                    objectguid__in=guids_list,
                    level__in=GAR_LEVELS_STREET,
                )

            else:
                typenames_data = AddrObj.objects.filter(
                    isactive=True,
                    level__in=GAR_LEVELS_STREET,
                )

            self.update_db_data(
                model=AddrObj,
                data_for_update=typenames_data,
                column_name='name_with_typename',
                fill_column_function=self.get_name_with_full_typename,
            )



def sort_models(items):
    """
    Производит топологическую сортировку django-моделей
    """

    def add_node(graph, node):
        graph.setdefault(node, [0])

    def add_arc(graph, fromnode, tonode):
        graph[fromnode].append(tonode)
        graph[tonode][0] += 1

    graph = {}

    for v in items:
        add_node(graph, v)

    for a in items:

        for field in a._meta.get_fields():
            if field.concrete:
                if field.many_to_many or field.many_to_one:
                    add_arc(graph, field.related_model, a)

    roots = [node for (node, nodeinfo) in graph.items() if nodeinfo[0] == 0]

    sorted = []
    while roots:
        root = roots.pop()
        sorted.append(root)
        for child in graph[root][1:]:
            graph[child][0] = graph[child][0] - 1
            if graph[child][0] == 0:
                roots.append(child)
        del graph[root]

    if len(graph.items()) != 0:
        raise CircularReferenceException

    return sorted


class CircularReferenceException(Exception):
    pass



from django.core.management import (
    BaseCommand,
)
from django.db import (
    connections,
    transaction,
)
from django.utils.functional import (
    partition,
)

from m3_gar import (
    config,
)
from m3_gar.importer.constraints import (
    ConstraintsEditor,
)
from m3_gar.importer.models_sort import (
    sort_models,
)
from m3_gar.models.util import (
    RegionCodeModelMixin,
)
from m3_gar.util import (
    get_models,
)


class Command(BaseCommand):
    help = 'Manage DB constraints and indexes'

    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument(
            'state',
            choices=['enable', 'disable'],
            help='Enable/disable database consistency/durability',
        )

        parser.add_argument(
            '--truncate',
            type=str,
            default=None,
            nargs='?',
            const='',
            help='Truncate all data (works only with "disable" state)',
        )
        parser.add_argument(
            '--fk',
            action='store_true',
            default=False,
            help='Perform operations for foreign key constraints',
        )
        parser.add_argument(
            '--unique',
            action='store_true',
            default=False,
            help='Perform operations for unique constraints',
        )
        parser.add_argument(
            '--index',
            action='store_true',
            default=False,
            help='Perform operations for database indexes',
        )
        parser.add_argument(
            '--logged',
            action='store_true',
            default=False,
            help='Change tables logging status',
        )
        parser.add_argument(
            '--commit',
            action='store_true',
            default=False,
            help='Commit produced SQL to the database',
        )

    def handle(
        self, *args,
        state,
        truncate,
        fk,
        unique,
        index,
        logged,
        commit,
        **kwargs,
    ):
        self.models = sort_models(get_models())
        self.sql_collected = []
        self.enable = state == 'enable'

        if not self.enable and truncate is not None:
            regions = truncate and truncate.split(',') or []
            regions = list(map(int, regions))

            self.truncate(regions)

        if any((fk, unique, index)):
            self.set_constraints(fk, unique, index)

        if logged:
            self.set_logged()

        for sql in self.sql_collected:
            self.stdout.write(sql)

        if commit:
            with transaction.atomic(using=config.DATABASE_ALIAS):
                conn = connections[config.DATABASE_ALIAS]
                with conn.cursor() as cursor:
                    for sql in self.sql_collected:
                        cursor.execute(sql)

    def truncate(self, regions):
        truncate_all = not regions
        truncate_dicts = 0 in regions

        regions_str = ','.join(str(region) for region in regions if region != 0)

        dicts, regional = partition(
            lambda m: issubclass(m, RegionCodeModelMixin),
            self.models,
        )

        if truncate_all:
            truncate_models = self.models
            delete_models = []
        else:
            if truncate_dicts:
                truncate_models = dicts
            else:
                truncate_models = []

            delete_models = regional

        for model in truncate_models:
            sql = f'TRUNCATE TABLE {model._meta.db_table} RESTART IDENTITY CASCADE;'
            self.sql_collected.append(sql)

        for model in delete_models:
            sql = f'DELETE FROM {model._meta.db_table} WHERE region_code IN ({regions_str});'
            self.sql_collected.append(sql)

    def set_constraints(self, fk, unique, index):
        constraints_editor = ConstraintsEditor(
            enable=self.enable,
            fk=fk,
            unique=unique,
            index=index,
        )

        for model in self.models:
            for sql in constraints_editor.change_model_constraints(model):
                self.sql_collected.extend(sql)

    def set_logged(self):
        logged = (
            'LOGGED'
            if self.enable
            else 'UNLOGGED'
        )

        for model in self.models:
            sql = f'ALTER TABLE {model._meta.db_table} SET {logged};'
            self.sql_collected.append(sql)



# Generated by Django 3.2.6 on 2022-02-17 06:39

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('m3_gar', '0008_auto_20220217_0556'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Status',
        ),
    ]
